'''module docstring'''

from textwatermark.defines import WMMethod

# 同形异义-字母
CONFUSABLES_CHARS = {
    'A': 'ΑАᎪꓮ𝐀𝖠𝗔',
    'B': 'ΒВᏴꓐ𝐁𝖡𝗕',
    'C': 'ⅭＣСᏟⲤꓚϹ',
    'D': 'ⅮᗞᗪᎠꓓ𝐃𝖣',
    'E': 'ⴹΕЕᎬꓰ𝐄𝖤',
    'F': 'ᖴϜꓝ𝐅𝖥𝙵𝟊',
    'G': 'Ꮐ𝙶ꓖ𝐆𝖦𝗚𝐺',
    'H': 'ᕼΗНᎻⲎꓧ𝖧',
    'I': 'ⅠΙІӀ𝐈𝙸𝚰',
    'J': 'ᒍЈᎫ𝐉𝙹𝖩𝗝',
    'K': 'KΚКᏦⲔꓗ𝐊',
    'L': 'ⅬᒪᏞⳐꓡ𝐋𝙻',
    'M': 'ⅯＭΜМᎷꓟ𝐌',
    'N': 'ＮΝⲚꓠ𝐍𝖭𝗡',
    'O': 'ଠⵔՕⲞꓳ𝚶𝐎',
    'P': 'ΡРᏢⲢꓑ𝐏𝖯',
    'Q': '𝐐𝑄𝑸𝖰𝗤𝘘𝙌',
    'R': 'ᎡᏒꓣ𝐑𝖱𝗥𝚁',
    'S': 'ЅՏᏚꓢ𝖲𝗦𝚂',
    'T': 'ΤТᎢꓔ𝖳𝗧𝚃',
    'U': 'Սꓴ𝚄𝐔𝖴𝗨𝘜',
    'V': 'ⅤᐯⴸᏙꓦ𝐕𝖵',
    'W': 'ԜᎳꓪ𝐖𝖶𝗪𝘞',
    'X': 'ΧХⲬⵝꓫ𐊐𐊴',
    'Y': 'ΥҮⲨꓬ𝐘𝖸𝗬',
    'Z': 'ΖᏃꓜꛉ𐕓𑢩Ⲍ',
    'a': 'аɑαᥲ⍺𐓟ꭤ',
    'b': 'ᖯЬᏏ𝐛𝖻𝗯𝚋',
    'c': 'ϲсᥴᴄⅽⲥ𐔛',
    'd': 'ⅾᑯꓒ𝐝𝖽𝗱𝙙',
    'e': 'еℯ𝐞𝖾𝗲𝘦𝙚',
    'f': 'ẝ𝐟𝕗𝖿𝗳𝙛𝘧',
    'g': 'ɡց𝐠𝑔𝒈𝗀𝗴',
    'h': 'һᏂ𝐡𝕙𝗁𝗵𝚑',
    'i': 'іᎥ𝐢𝕚𝗂𝗶𝚒',
    'j': 'ј𝚓ϳ𝐣𝕛𝗃𝗷',
    'k': '𝐤𝗄𝗸𝙠𝚔𝑘𝒌',
    'l': '𝑙𝒍𝓁𝓵𝗅𝗹𝙡',
    'm': 'ⅿ𝐦𝗆𝗺𝘮𝙢𝑚',
    'n': 'ո𝐧𝗇𝗻𝚗𝘯𝙣',
    'o': 'οоᴏⲟ𐓪օ௦',
    'p': 'рⲣ𝐩𝗉𝗽𝚙𝑝',
    'q': 'ԛգզ𝐪𝗊𝗾𝚚',
    'r': '𝚛𝐫𝒓𝓇𝓻𝗿𝗋',
    's': '᥉ѕꮪꜱ𝚜𝐬𝗌',
    't': '𝚝𝑡𝓉𝓽𝗍𝘵𝙩',
    'u': 'ᴜս𝐮𝗎𝘂𝚞𝑢',
    'v': 'ᴠⅴνѵ𝐯∨𝝼',
    'w': 'ᴡԝ𝐰𝗐𝘄𝘸𝙬',
    'x': 'ⅹх𝐱᙮𝗑𝘅𝘹',
    'y': 'уꭹ𝐲𝒚𝗒𝘆𝚢',
    'z': 'ᴢ𝐳𝑧𝗓𝘇𝘻𝙯',
}

for ikey, ival in CONFUSABLES_CHARS.items():
    CONFUSABLES_CHARS[ikey] = ikey + ival
method = WMMethod.FIND_AND_REPLACE
