'''module docstring'''

from textwatermark.defines import WMMethod

# 空格字符
CONFUSABLES_CHARS = [
    '<a></a>',
    '<abbr></abbr>',
    '<b></b>',
    '<bdo></bdo>',
    '<big></big>',
    '<cite></cite>',
    '<code></code>',
    '<dfn></dfn>',
    '<em></em>',
    '<i></i>',
    '<kbd></kbd>',
    '<var></var>',
    '<samp></samp>',
    '<span></span>',
    '<strong></strong>',
    '<small></small>',
    '<sub></sub>',
    '<sup></sup>',
    '<tt></tt>',
    '<u></u>',
]

method = WMMethod.INSERT_INTO_POSITION
